"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const Mark_1 = __importDefault(require("./Mark"));
const SelectionToolbar_1 = require("../components/SelectionToolbar");
function markApplies(doc, ranges, type) {
    for (let i = 0; i < ranges.length; i++) {
        const { $from, $to } = ranges[i];
        let can = $from.depth === 0 ? doc.type.allowsMarkType(type) : false;
        doc.nodesBetween($from.pos, $to.pos, node => {
            if (can)
                return false;
            can = node.inlineContent && node.type.allowsMarkType(type);
        });
        if (can)
            return true;
    }
    return false;
}
const LINK_INPUT_REGEX = /\[(.+)]\((\S+)\)/;
function isPlainURL(link, parent, index, side) {
    if (link.attrs.title || !/^\w+:/.test(link.attrs.href)) {
        return false;
    }
    const content = parent.child(index + (side < 0 ? -1 : 0));
    if (!content.isText ||
        content.text !== link.attrs.href ||
        content.marks[content.marks.length - 1] !== link) {
        return false;
    }
    if (index === (side < 0 ? 1 : parent.childCount - 1)) {
        return true;
    }
    const next = parent.child(index + (side < 0 ? -2 : 1));
    return !link.isInSet(next.marks);
}
class Link extends Mark_1.default {
    get name() {
        return "link";
    }
    get schema() {
        return {
            attrs: {
                href: {
                    default: "",
                },
            },
            inclusive: false,
            parseDOM: [
                {
                    tag: "a[href]",
                    getAttrs: (dom) => ({
                        href: dom.getAttribute("href"),
                    }),
                },
            ],
            toDOM: node => [
                "a",
                Object.assign(Object.assign({}, node.attrs), { rel: "noopener noreferrer nofollow" }),
                0,
            ],
        };
    }
    inputRules({ type }) {
        return [
            new prosemirror_inputrules_1.InputRule(LINK_INPUT_REGEX, (state, match, start, end) => {
                const [okay, alt, href] = match;
                const { tr } = state;
                if (okay) {
                    tr.replaceWith(start, end, this.editor.schema.text(alt)).addMark(start, start + alt.length, type.create({ href }));
                }
                return tr;
            }),
        ];
    }
    commands({ type }) {
        return ({ href } = { href: "" }) => {
            return (state, dispatch) => {
                const { empty, $cursor, ranges } = state.selection;
                if ((empty && !$cursor) || !markApplies(state.doc, ranges, type))
                    return false;
                if (empty) {
                    this.options.onKeyboardShortcut();
                }
                if (dispatch) {
                    if ($cursor) {
                        if (type.isInSet(state.storedMarks || $cursor.marks()))
                            dispatch(state.tr.removeStoredMark(type));
                        else
                            dispatch(state.tr.addStoredMark(type.create({ href })));
                    }
                    else {
                        let has = false, tr = state.tr;
                        for (let i = 0; !has && i < ranges.length; i++) {
                            const { $from, $to } = ranges[i];
                            has = state.doc.rangeHasMark($from.pos, $to.pos, type);
                        }
                        for (let i = 0; i < ranges.length; i++) {
                            const { $from, $to } = ranges[i];
                            if (has)
                                tr.removeMark($from.pos, $to.pos, type);
                            else
                                tr.addMark($from.pos, $to.pos, type.create({ href }));
                        }
                        dispatch(tr.scrollIntoView());
                    }
                }
                return true;
            };
        };
    }
    keys({ type }) {
        return {
            "Mod-k": (state, dispatch) => {
                if (state.selection.empty) {
                    this.options.onKeyboardShortcut();
                    return true;
                }
                return prosemirror_commands_1.toggleMark(type, { href: "" })(state, dispatch);
            },
        };
    }
    get plugins() {
        const onStart = (view, event) => {
            if (event.target instanceof HTMLAnchorElement) {
                window.TOUCHLINKMOVED = false;
                return true;
            }
            return false;
        };
        const onMove = (view, event) => {
            if (event.target instanceof HTMLAnchorElement &&
                !window.TOUCHLINKMOVED) {
                window.TOUCHLINKMOVED = true;
                return true;
            }
            return false;
        };
        const onEnd = (view, event, preventDefault = false) => {
            var _a, _b;
            if (event.target instanceof HTMLAnchorElement &&
                !window.TOUCHLINKMOVED) {
                preventDefault && event.preventDefault();
                const { href } = event.target;
                const selectionContent = (_b = (_a = view === null || view === void 0 ? void 0 : view.state) === null || _a === void 0 ? void 0 : _a.selection) === null || _b === void 0 ? void 0 : _b.content();
                const selectedText = selectionContent && SelectionToolbar_1.getText(selectionContent);
                if (!selectedText) {
                    const isHashtag = href.startsWith("#");
                    if (isHashtag && this.options.onClickHashtag) {
                        this.options.onClickHashtag(href, event);
                        return true;
                    }
                    if (this.options.onClickLink) {
                        this.options.onClickLink(href, event);
                        return true;
                    }
                }
            }
            return false;
        };
        return [
            new prosemirror_state_1.Plugin({
                props: {
                    handleDOMEvents: {
                        touchstart: onStart,
                        mousedown: onStart,
                        touchmove: onMove,
                        mousemove: onMove,
                        touchend: (view, event) => onEnd(view, event, true),
                        mouseup: (view, event) => onEnd(view, event),
                        mouseover: (view, event) => {
                            if (event.target instanceof HTMLAnchorElement &&
                                !event.target.className.includes("ProseMirror-widget")) {
                                if (this.options.onHoverLink) {
                                    return this.options.onHoverLink(event);
                                }
                            }
                            return false;
                        },
                        click: (view, event) => {
                            if (event.target instanceof HTMLAnchorElement) {
                                if (this.options.onClickLink) {
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                                return true;
                            }
                            return false;
                        },
                    },
                },
            }),
        ];
    }
    get toMarkdown() {
        return {
            open(_state, mark, parent, index) {
                return isPlainURL(mark, parent, index, 1) ? "<" : "[";
            },
            close(state, mark, parent, index) {
                return isPlainURL(mark, parent, index, -1)
                    ? ">"
                    : "](" +
                        state.esc(mark.attrs.href) +
                        (mark.attrs.title ? " " + state.quote(mark.attrs.title) : "") +
                        ")";
            },
        };
    }
    parseMarkdown() {
        return {
            mark: "link",
            getAttrs: tok => ({
                href: tok.attrGet("href"),
                title: tok.attrGet("title") || null,
            }),
        };
    }
}
exports.default = Link;
//# sourceMappingURL=Link.js.map